@@ -168,7 +168,7 @@ def _has_namedtuple_base(node):
     return set(node.basenames) & TYPING_NAMEDTUPLE_BASENAMES
 
 
-def _looks_like(node, name):
+def _looks_like(node, name) -> bool:
     func = node.func
     if isinstance(func, nodes.Attribute):
         return func.attrname == name
@@ -193,7 +193,6 @@ def infer_named_tuple(
     call_site = arguments.CallSite.from_call(node, context=context)
     node = extract_node("import collections; collections.namedtuple")
     try:
-
         func = next(node.infer())
     except StopIteration as e:
         raise InferenceError(node=node) from e
@@ -425,6 +424,10 @@ def infer_enum_class(node: nodes.ClassDef) -> nodes.ClassDef:
                 new_targets.append(fake.instantiate_class())
                 dunder_members[local] = fake
             node.locals[local] = new_targets
+
+        # The undocumented `_value2member_map_` member:
+        node.locals["_value2member_map_"] = [nodes.Dict(parent=node)]
+
         members = nodes.Dict(parent=node)
         members.postinit(
             [
@@ -460,7 +463,7 @@ def infer_enum_class(node: nodes.ClassDef) -> nodes.ClassDef:
     return node
 
 
-def infer_typing_namedtuple_class(class_node, context=None):
+def infer_typing_namedtuple_class(class_node, context: InferenceContext | None = None):
     """Infer a subclass of typing.NamedTuple"""
     # Check if it has the corresponding bases
     annassigns_fields = [
@@ -493,7 +496,7 @@ def infer_typing_namedtuple_class(class_node, context=None):
     return iter((generated_class_node,))
 
 
-def infer_typing_namedtuple_function(node, context=None):
+def infer_typing_namedtuple_function(node, context: InferenceContext | None = None):
     """
     Starting with python3.9, NamedTuple is a function of the typing module.
     The class NamedTuple is build dynamically through a call to `type` during
