@@ -6,6 +6,9 @@
 Various helper utilities.
 """
 
+from __future__ import annotations
+
+from collections.abc import Generator
 
 from astroid import bases, manager, nodes, raw_building, util
 from astroid.context import CallContext, InferenceContext
@@ -17,15 +20,18 @@ from astroid.exceptions import (
     _NonDeducibleTypeHierarchy,
 )
 from astroid.nodes import scoped_nodes
+from astroid.typing import InferenceResult, SuccessfulInferenceResult
 
 
-def _build_proxy_class(cls_name, builtins):
+def _build_proxy_class(cls_name: str, builtins: nodes.Module) -> nodes.ClassDef:
     proxy = raw_building.build_class(cls_name)
     proxy.parent = builtins
     return proxy
 
 
-def _function_type(function, builtins):
+def _function_type(
+    function: nodes.Lambda | bases.UnboundMethod, builtins: nodes.Module
+) -> nodes.ClassDef:
     if isinstance(function, scoped_nodes.Lambda):
         if function.root().name == "builtins":
             cls_name = "builtin_function_or_method"
@@ -33,12 +39,14 @@ def _function_type(function, builtins):
             cls_name = "function"
     elif isinstance(function, bases.BoundMethod):
         cls_name = "method"
-    elif isinstance(function, bases.UnboundMethod):
+    else:
         cls_name = "function"
     return _build_proxy_class(cls_name, builtins)
 
 
-def _object_type(node, context=None):
+def _object_type(
+    node: SuccessfulInferenceResult, context: InferenceContext | None = None
+) -> Generator[InferenceResult | None, None, None]:
     astroid_manager = manager.AstroidManager()
     builtins = astroid_manager.builtins_module
     context = context or InferenceContext()
@@ -57,11 +65,17 @@ def _object_type(node, context=None):
             yield _build_proxy_class("module", builtins)
         elif isinstance(inferred, nodes.Unknown):
             raise InferenceError
-        else:
+        elif inferred is util.Uninferable:
+            yield inferred
+        elif isinstance(inferred, (bases.Proxy, nodes.Slice)):
             yield inferred._proxied
+        else:  # pragma: no cover
+            raise AssertionError(f"We don't handle {type(inferred)} currently")
 
 
-def object_type(node, context=None):
+def object_type(
+    node: SuccessfulInferenceResult, context: InferenceContext | None = None
+) -> InferenceResult | None:
     """Obtain the type of the given node
 
     This is used to implement the ``type`` builtin, which means that it's
@@ -80,7 +94,9 @@ def object_type(node, context=None):
     return list(types)[0]
 
 
-def _object_type_is_subclass(obj_type, class_or_seq, context=None):
+def _object_type_is_subclass(
+    obj_type, class_or_seq, context: InferenceContext | None = None
+):
     if not isinstance(class_or_seq, (tuple, list)):
         class_seq = (class_or_seq,)
     else:
@@ -107,7 +123,7 @@ def _object_type_is_subclass(obj_type, class_or_seq, context=None):
     return False
 
 
-def object_isinstance(node, class_or_seq, context=None):
+def object_isinstance(node, class_or_seq, context: InferenceContext | None = None):
     """Check if a node 'isinstance' any node in class_or_seq
 
     :param node: A given node
@@ -122,7 +138,7 @@ def object_isinstance(node, class_or_seq, context=None):
     return _object_type_is_subclass(obj_type, class_or_seq, context=context)
 
 
-def object_issubclass(node, class_or_seq, context=None):
+def object_issubclass(node, class_or_seq, context: InferenceContext | None = None):
     """Check if a type is a subclass of any node in class_or_seq
 
     :param node: A given node
@@ -138,7 +154,9 @@ def object_issubclass(node, class_or_seq, context=None):
     return _object_type_is_subclass(node, class_or_seq, context=context)
 
 
-def safe_infer(node, context=None):
+def safe_infer(
+    node: nodes.NodeNG | bases.Proxy, context: InferenceContext | None = None
+) -> InferenceResult | None:
     """Return the inferred value for the given node.
 
     Return None if inference failed or if there is some ambiguity (more than
@@ -158,8 +176,8 @@ def safe_infer(node, context=None):
         return value
 
 
-def has_known_bases(klass, context=None):
-    """Return true if all base classes of a class could be inferred."""
+def has_known_bases(klass, context: InferenceContext | None = None) -> bool:
+    """Return whether all base classes of a class could be inferred."""
     try:
         return klass._all_bases_known
     except AttributeError:
@@ -178,7 +196,7 @@ def has_known_bases(klass, context=None):
     return True
 
 
-def _type_check(type1, type2):
+def _type_check(type1, type2) -> bool:
     if not all(map(has_known_bases, (type1, type2))):
         raise _NonDeducibleTypeHierarchy
 
@@ -191,17 +209,17 @@ def _type_check(type1, type2):
         raise _NonDeducibleTypeHierarchy from e
 
 
-def is_subtype(type1, type2):
+def is_subtype(type1, type2) -> bool:
     """Check if *type1* is a subtype of *type2*."""
     return _type_check(type1=type2, type2=type1)
 
 
-def is_supertype(type1, type2):
+def is_supertype(type1, type2) -> bool:
     """Check if *type2* is a supertype of *type1*."""
     return _type_check(type1, type2)
 
 
-def class_instance_as_index(node):
+def class_instance_as_index(node: SuccessfulInferenceResult) -> nodes.Const | None:
     """Get the value as an index for the given instance.
 
     If an instance provides an __index__ method, then it can
@@ -224,7 +242,7 @@ def class_instance_as_index(node):
     return None
 
 
-def object_len(node, context=None):
+def object_len(node, context: InferenceContext | None = None):
     """Infer length of given node object
 
     :param Union[nodes.ClassDef, nodes.Instance] node:
