@@ -18,7 +18,7 @@ from collections.abc import Iterable
 from typing import Any, Union
 
 from astroid import bases, nodes
-from astroid.const import IS_PYPY
+from astroid.const import _EMPTY_OBJECT_MARKER, IS_PYPY
 from astroid.manager import AstroidManager
 from astroid.nodes import node_classes
 
@@ -39,12 +39,12 @@ TYPE_NOTIMPLEMENTED = type(NotImplemented)
 TYPE_ELLIPSIS = type(...)
 
 
-def _attach_local_node(parent, node, name):
+def _attach_local_node(parent, node, name: str) -> None:
     node.name = name  # needed by add_local_node
     parent.add_local_node(node)
 
 
-def _add_dunder_class(func, member):
+def _add_dunder_class(func, member) -> None:
     """Add a __class__ member to the given func node, if we can determine it."""
     python_cls = member.__class__
     cls_name = getattr(python_cls, "__name__", None)
@@ -55,10 +55,7 @@ def _add_dunder_class(func, member):
     func.instance_attrs["__class__"] = [ast_klass]
 
 
-_marker = object()
-
-
-def attach_dummy_node(node, name, runtime_object=_marker):
+def attach_dummy_node(node, name: str, runtime_object=_EMPTY_OBJECT_MARKER) -> None:
     """create a dummy node and register it in the locals of the given
     node with the specified name
     """
@@ -67,14 +64,7 @@ def attach_dummy_node(node, name, runtime_object=_marker):
     _attach_local_node(node, enode, name)
 
 
-def _has_underlying_object(self):
-    return self.object is not None and self.object is not _marker
-
-
-nodes.EmptyNode.has_underlying_object = _has_underlying_object
-
-
-def attach_const_node(node, name, value):
+def attach_const_node(node, name: str, value) -> None:
     """create a Const node and register it in the locals of the given
     node with the specified name
     """
@@ -82,7 +72,7 @@ def attach_const_node(node, name, value):
         _attach_local_node(node, nodes.const_factory(value), name)
 
 
-def attach_import_node(node, modname, membername):
+def attach_import_node(node, modname: str, membername: str) -> None:
     """create a ImportFrom node and register it in the locals of the given
     node with the specified name
     """
@@ -135,9 +125,18 @@ def build_function(
     else:
         arguments = None
 
+    default_nodes: list[nodes.NodeNG] | None = []
+    if defaults is not None:
+        for default in defaults:
+            default_node = nodes.const_factory(default)
+            default_node.parent = argsnode
+            default_nodes.append(default_node)
+    else:
+        default_nodes = None
+
     argsnode.postinit(
         args=arguments,
-        defaults=[],
+        defaults=default_nodes,
         kwonlyargs=[
             nodes.AssignName(name=arg, parent=argsnode) for arg in kwonlyargs or ()
         ],
@@ -152,31 +151,32 @@ def build_function(
         body=[],
         doc_node=nodes.Const(value=doc) if doc else None,
     )
-    for default in defaults or ():
-        argsnode.defaults.append(nodes.const_factory(default))
-        argsnode.defaults[-1].parent = argsnode
     if args:
         register_arguments(func)
     return func
 
 
-def build_from_import(fromname, names):
+def build_from_import(fromname: str, names: list[str]) -> nodes.ImportFrom:
     """create and initialize an astroid ImportFrom import statement"""
     return nodes.ImportFrom(fromname, [(name, None) for name in names])
 
 
-def register_arguments(func, args=None):
+def register_arguments(func: nodes.FunctionDef, args: list | None = None) -> None:
     """add given arguments to local
 
     args is a list that may contains nested lists
     (i.e. def func(a, (b, c, d)): ...)
     """
+    # If no args are passed in, get the args from the function.
     if args is None:
-        args = func.args.args
         if func.args.vararg:
             func.set_local(func.args.vararg, func.args)
         if func.args.kwarg:
             func.set_local(func.args.kwarg, func.args)
+        args = func.args.args
+        # If the function has no args, there is nothing left to do.
+        if args is None:
+            return
     for arg in args:
         if isinstance(arg, nodes.AssignName):
             func.set_local(arg.name, arg)
@@ -194,7 +194,7 @@ def object_build_class(
 
 def _get_args_info_from_callable(
     member: _FunctionTypes,
-) -> tuple[list[str], list[Any], list[str], list[str]]:
+) -> tuple[list[str], list[str], list[Any], list[str]]:
     """Returns args, posonlyargs, defaults, kwonlyargs.
 
     :note: currently ignores the return annotation.
@@ -324,13 +324,6 @@ def _build_from_function(
         object_build_function(node, member, name)
 
 
-def _safe_has_attribute(obj, member):
-    try:
-        return hasattr(obj, member)
-    except Exception:  # pylint: disable=broad-except
-        return False
-
-
 class InspectBuilder:
     """class for building nodes from living object
 
@@ -338,7 +331,7 @@ class InspectBuilder:
     FunctionDef and ClassDef nodes and some others as guessed.
     """
 
-    def __init__(self, manager_instance=None):
+    def __init__(self, manager_instance: AstroidManager | None = None) -> None:
         self._manager = manager_instance or AstroidManager()
         self._done: dict[types.ModuleType | type, nodes.Module | nodes.ClassDef] = {}
         self._module: types.ModuleType
@@ -388,9 +381,9 @@ class InspectBuilder:
             pypy__class_getitem__ = IS_PYPY and name == "__class_getitem__"
             try:
                 with warnings.catch_warnings():
-                    warnings.simplefilter("error")
+                    warnings.simplefilter("ignore")
                     member = getattr(obj, name)
-            except (AttributeError, DeprecationWarning):
+            except AttributeError:
                 # damned ExtensionClass.Base, I know you're there !
                 attach_dummy_node(node, name)
                 continue
@@ -426,7 +419,7 @@ class InspectBuilder:
                 # This should be called for Jython, where some builtin
                 # methods aren't caught by isbuiltin branch.
                 _build_from_function(node, name, member, self._module)
-            elif _safe_has_attribute(member, "__all__"):
+            elif hasattr(member, "__all__"):
                 module = build_module(name)
                 _attach_local_node(node, module, name)
                 # recursion
@@ -481,12 +474,12 @@ class InspectBuilder:
 _CONST_PROXY: dict[type, nodes.ClassDef] = {}
 
 
-def _set_proxied(const):
+def _set_proxied(const) -> nodes.ClassDef:
     # TODO : find a nicer way to handle this situation;
     return _CONST_PROXY[const.value.__class__]
 
 
-def _astroid_bootstrapping():
+def _astroid_bootstrapping() -> None:
     """astroid bootstrapping the builtins module"""
     # this boot strapping is necessary since we need the Const nodes to
     # inspect_build builtins, and then we can proxy Const
